# -*- coding: utf-8 -*-
"""Python client for the RESTful API of the Propertyshelf MLS."""


HTTP_HEADER_PREFIX = 'X-MLS'
PRODUCT_NAME = 'mls.apiclient'
REST_API_URL = 'api/rest'
REST_API_VERSION = 'v1'
